local SharedMacrosAddon = LibStub("AceAddon-3.0"):NewAddon("SharedMacros")
SharedMacrosAddon.ADDON_NAME = "SharedMacros"
SharedMacrosAddon.VERSION = GetAddOnMetadata("SharedMacros", "Version")

local defaults = {
  global = {
    cleanse = {
		enabled = true,
		name = "Cleanse",
		body = "#showtooltip\n/tar [@mouseover,help,nodead] [@target,help,nodead] [@targettarget,help,nodead] [@player]\n/use CLEANSE_SPELL\n/targetlasttarget"
	},
	purge = {
		enabled = true,
		name = "Purge",
		body = "#showtooltip PURGE_SPELL\n/use [@target,harm,nodead] [@targettarget,harm,nodead] PURGE_SPELL"
	},
	dispel = {
		enabled = true,
		name = "Dispel",
		body = "#showtooltip [@mouseover,help,nodead] CLEANSE_SPELL;PURGE_SPELL\n/use [@mouseover,help,nodead] CLEANSE_SPELL;[@target,harm,nodead] PURGE_SPELL; [@target,help,nodead] CLEANSE_SPELL"
	},
	interrupt = {
		enabled = true,
		name = "Interrupt",
		body = "#showtooltip\n/stopcasting\n/use INTERRUPT_SPELL"
	},
	resurrection = {
		enabled = true,
		name = "CR",
		body = "#showtooltip RESURRECTION_SPELL\n/stopcasting\n/use [@mouseover,help,dead][help,dead] RESURRECTION_SPELL"
	},
	sprint = {
		enabled = true,
		name = "Sprint",
		body = "#showtooltip SPRINT_SPELL\n/stopcasting\n/use SPRINT_SPELL"
	},
	taunt = {
		enabled = true,
		name = "Taunt",
		body = "#showtooltip TAUNT_SPELL\n/use TAUNT_SPELL"
	},
  }
}

function SharedMacrosAddon:OnInitialize()
	if (IsAddOnLoaded("DispelMacro")) then
		print("DispelMacro has been renamed to SharedMacros. Please remove the DispelMacro folder from your AddOns folder!")
		return
	end
	self.db = LibStub("AceDB-3.0"):New("DispelMacroDB", defaults) -- keep the old DB name, otherwise people may lose their settings
	
	LibStub("AceEvent-3.0").RegisterEvent(SharedMacrosAddon, "SPELLS_CHANGED", function(e) SharedMacrosAddon:Main(); end)
	LibStub("LibAboutPanel").new(nil, SharedMacrosAddon.ADDON_NAME)
	LibStub("AceConfigRegistry-3.0").RegisterOptionsTable(SharedMacrosAddon, SharedMacrosAddon.ADDON_NAME.."-Config", function() return SharedMacrosAddon:GetConfigOptionsTable() end)
	LibStub("AceConfigDialog-3.0").AddToBlizOptions(SharedMacrosAddon, SharedMacrosAddon.ADDON_NAME.."-Config", "Config", SharedMacrosAddon.ADDON_NAME)
end

function SharedMacrosAddon:Main()
	local function GetPlayerSpell(arr)
		for _,v in pairs(arr) do
			if(IsPlayerSpell(v)) then
				return select(1, GetSpellInfo(v))
			end
		end
		return nil
	end
	
	local function GetPetSpell(arr)
		for _,v in pairs(arr) do
			if(IsSpellKnown(v, true)) then
				return select(1, GetSpellInfo(v))
			end
		end
		return nil
	end

	if InCombatLockdown() then return end
	
	local cleanseName = GetPlayerSpell(_G["DM_CLEANSE_SPELL_ARRAY"])
	local purgeName = GetPlayerSpell(_G["DM_PURGE_SPELL_ARRAY"])
	local interruptName = GetPlayerSpell(_G["DM_INTERRUPT_SPELL_ARRAY"]) or GetPetSpell(_G["DM_INTERRUPT_SPELL_ARRAY"])
	local resurrectionName = GetPlayerSpell(_G["DM_COMBATRESURRECTION_SPELL_ARRAY"])
	local sprintName = GetPlayerSpell(_G["DM_SPRINT_SPELL_ARRAY"])
	local tauntName = GetPlayerSpell(_G["DM_TAUNT_SPELL_ARRAY"]) or GetPetSpell(_G["DM_TAUNT_SPELL_ARRAY"])
	if (self.db.global.cleanse.enabled and cleanseName ~= nil) then 
		SharedMacrosAddon:DoCreateMacro(self.db.global.cleanse.body, {["CLEANSE_SPELL"] = cleanseName}, self.db.global.cleanse.name) 
	end
	if (self.db.global.purge.enabled and purgeName ~= nil) then 
		SharedMacrosAddon:DoCreateMacro(self.db.global.purge.body, {["PURGE_SPELL"] = purgeName}, self.db.global.purge.name) 
	end
	if (self.db.global.dispel.enabled and cleanseName ~= nil and purgeName ~= nil) then 
		SharedMacrosAddon:DoCreateMacro(self.db.global.dispel.body, {["CLEANSE_SPELL"] = cleanseName, ["PURGE_SPELL"] = purgeName}, self.db.global.dispel.name) 
	end
	if (self.db.global.interrupt.enabled and interruptName) then 
		SharedMacrosAddon:DoCreateMacro(self.db.global.interrupt.body, {["INTERRUPT_SPELL"] = interruptName}, self.db.global.interrupt.name) 
	end
	if (self.db.global.resurrection.enabled and resurrectionName) then 
		SharedMacrosAddon:DoCreateMacro(self.db.global.resurrection.body, {["RESURRECTION_SPELL"] = resurrectionName}, self.db.global.resurrection.name) 
	end
	if (self.db.global.sprint.enabled and sprintName) then 
		SharedMacrosAddon:DoCreateMacro(self.db.global.sprint.body, {["SPRINT_SPELL"] = sprintName}, self.db.global.sprint.name) 
	end
	if (self.db.global.taunt.enabled and tauntName) then 
		SharedMacrosAddon:DoCreateMacro(self.db.global.taunt.body, {["TAUNT_SPELL"] = tauntName}, self.db.global.taunt.name) 
	end
end

local function trim(s)
	local txt = s
	if(txt ~= nil) then
		txt = txt:gsub("^%s*(.-)%s*$", "%1")
	  
		-- trim \n
		-- there must be some better way no?
		while(string.byte(txt, string.len(txt)) == 10) do
			txt = trim(string.sub(txt, 0, string.len(txt)-1))
		end
	end
	return txt
end

function SharedMacrosAddon:DoCreateMacro(body, replacements, macroName)
	if(string.len(body)>0) then
		local str = ""
		local replacedBody = body
		local originalBody = trim(GetMacroBody(macroName))
		for needle, spellName in pairs(replacements) do
			replacedBody = string.gsub(replacedBody, needle, spellName)
			if (str:len()>0) then str = str .. "and " end
			str = str .. "'" .. spellName .. "' "
		end
		replacedBody = trim(replacedBody)
		
		if(originalBody ~= nil and not (replacedBody == originalBody)) then
			print("SharedMacros: Edited macro '" .. macroName .. "' using " .. str)
			EditMacro(macroName, macroName, "INV_MISC_QUESTIONMARK", replacedBody, nil)
		elseif(originalBody == nil) then
			print("SharedMacros: Created macro '".. macroName .. "' using " .. str)
			CreateMacro(macroName, "INV_MISC_QUESTIONMARK", replacedBody, nil)
		end	
	end
end

function SharedMacrosAddon:GetOneOptionTable(num, spell, opt, desc, replacements)
	local str = ""
	for _, txt in pairs(replacements) do
		if (str:len()>0) then str = str .. " or " end
		str = str .. txt
	end
	return {
		header = {
			name = spell,
			type = "header",
			order = (num*10)+1,
		},
		desc = {
			name = desc,
			type = "description",
			order = (num*10)+2,
			width = "full",
		},
		enable = {
			name = "Enable "..spell,
			type = "toggle",
			order = (num*10)+3,
			width = "full",
			set = function(info,val) 
						opt.enabled = val
						if(opt.enabled) then
							SharedMacrosAddon:Main()
						end
					end,
			get = function() return opt.enabled end,
		},
		name = {
			name = spell.." macro name",
			type = "input",
			order = (num*10)+4,
			set = function(info,val) 
						if(string.len(val) > 0) then 
							opt.name = val 
							SharedMacrosAddon:Main()
						end
					end,
			get = function() return opt.name end,
			disabled = function() return not opt.enabled end,
		},
		body = {
			name = spell.." macro body",
			type = "input",
			multiline = 6,
			width = "full",
			order = (num*10)+5,
			set = function(info,val) opt.body = val; SharedMacrosAddon:Main(); end,
			get = function() return opt.body end,
			disabled = function() return not opt.enabled end,
		},
		hint = {
			name = "Hint: Use "..str.." where you want to enter the actual spell name.",
			type = "description",
			order = (num*10)+6,
		},
	}
end


function SharedMacrosAddon:GetConfigOptionsTable()
	return {
        name = "Options",
        type = "group",
        order = 1,
        args = {
			cleanse = { order = 1, type = "group", name = "Cleanse", args = SharedMacrosAddon:GetOneOptionTable(0, "Cleanse", self.db.global.cleanse, "Macro for Cleanse (defensive)", {"CLEANSE_SPELL"})},
			purge = { order = 2, type = "group", name = "Purge", args = SharedMacrosAddon:GetOneOptionTable(0, "Purge", self.db.global.purge, "Macro for Purge (offensive)", {"PURGE_SPELL"})},
			dispel = { order = 3, type = "group", name = "Dispel", args = SharedMacrosAddon:GetOneOptionTable(0, "Dispel", self.db.global.dispel, "Macro for Dispel (both defensive and offensive)", {"CLEANSE_SPELL", "PURGE_SPELL"})},
			interrupt = { order = 4, type = "group", name = "Interrupt", args = SharedMacrosAddon:GetOneOptionTable(0, "Interrupt", self.db.global.interrupt, "Macro for Interrupt", {"INTERRUPT_SPELL"})},
			resurrection = { order = 5, type = "group", name = "Combat Resurrection", args = SharedMacrosAddon:GetOneOptionTable(0, "Combat Resurrection", self.db.global.resurrection, "Macro for Combat Resurrection", {"RESURRECTION_SPELL"})},
			sprint = { order = 6, type = "group", name = "Sprint", args = SharedMacrosAddon:GetOneOptionTable(0, "Sprint", self.db.global.sprint, "Macro for Sprint", {"SPRINT_SPELL"})},
			taunt = { order = 7, type = "group", name = "Taunt", args = SharedMacrosAddon:GetOneOptionTable(0, "Taunt", self.db.global.taunt, "Macro for Taunt", {"TAUNT_SPELL"})},
		},
    }
end
